@@ -1,8 +1,8 @@
 /*****************************************************************************
  * multicat.c: netcat-equivalent for multicast
  *****************************************************************************
- * Copyright (C) 2009, 2011-2012, 2015 VideoLAN
- * $Id: multicat.c 68 2015-08-01 14:31:29Z massiot $
+ * Copyright (C) 2009, 2011-2012, 2015-2016 VideoLAN
+ * $Id$
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -44,6 +44,10 @@
 #include <sys/ioctl.h>
 #include <syslog.h>
 
+#ifdef __FreeBSD__
+#   include <sys/uio.h>
+#endif
+
 #ifdef SIOCGSTAMPNS
 #   define HAVE_TIMESTAMPS
 #endif
@@ -81,6 +85,8 @@ static bool b_input_udp = false, b_output_udp = false;
 static size_t i_asked_payload_size = DEFAULT_PAYLOAD_SIZE;
 static size_t i_rtp_header_size = RTP_HEADER_SIZE;
 static uint64_t i_rotate_size = DEFAULT_ROTATE_SIZE;
+static uint64_t i_rotate_offset = DEFAULT_ROTATE_OFFSET;
+static uint64_t i_duration = 0;
 static struct udprawpkt pktheader;
 static bool b_raw_packets = false;
 static uint8_t *pi_pid_cc_table = NULL;
@@ -104,7 +110,7 @@ static void (*pf_ExitWrite)(void);
 
 static void usage(void)
 {
-    msg_Raw( NULL, "Usage: multicat [-i <RT priority>] [-l <syslogtag>] [-t <ttl>] [-X] [-T <file name>] [-f] [-p <PCR PID>] [-C] [-P] [-s <chunks>] [-n <chunks>] [-k <start time>] [-d <duration>] [-a] [-r <file duration>] [-S <SSRC IP>] [-u] [-U] [-m <payload size>] [-R <RTP header size>] [-w] <input item> <output item>" );
+    msg_Raw( NULL, "Usage: multicat [-i <RT priority>] [-l <syslogtag>] [-t <ttl>] [-X] [-T <file name>] [-f] [-p <PCR PID>] [-C] [-P] [-s <chunks>] [-n <chunks>] [-k <start time>] [-d <duration>] [-a] [-r <file duration>] [-O <rotate offset>] [-S <SSRC IP>] [-u] [-U] [-m <payload size>] [-R <RTP header size>] [-w] <input item> <output item>" );
     msg_Raw( NULL, "    item format: <file path | device path | FIFO path | directory path | network host>" );
     msg_Raw( NULL, "    host format: [<connect addr>[:<connect port>]][@[<bind addr][:<bind port>]]" );
     msg_Raw( NULL, "    -X: also pass-through all packets to stdout" );
@@ -119,6 +125,7 @@ static void usage(void)
     msg_Raw( NULL, "    -d: exit after definite time (in 27 MHz units)" );
     msg_Raw( NULL, "    -a: append to existing destination file (risky)" );
     msg_Raw( NULL, "    -r: in directory mode, rotate file after this duration (default: 97200000000 ticks = 1 hour)" );
+    msg_Raw( NULL, "    -O: in directory mode, rotate file after duration + this offset (default: 0 tick = calendar hour)" );
     msg_Raw( NULL, "    -S: overwrite or create RTP SSRC" );
     msg_Raw( NULL, "    -u: source has no RTP header" );
     msg_Raw( NULL, "    -U: destination has no RTP header" );
@@ -276,13 +283,17 @@ static ssize_t raw_Write( const void *p_buf, size_t i_len )
     ssize_t i_ret;
     struct iovec iov[2];
 
-    #ifdef __FAVOR_BSD
+    #if defined(__FreeBSD__)
     pktheader.udph.uh_ulen
     #else
     pktheader.udph.len
     #endif
     = htons(sizeof(struct udphdr) + i_len);
 
+    #if defined(__FreeBSD__)
+    pktheader.iph.ip_len = htons(sizeof(struct udprawpkt) + i_len);
+    #endif
+
     iov[0].iov_base = &pktheader;
     iov[0].iov_len = sizeof(struct udprawpkt);
 
@@ -353,6 +364,7 @@ static int udp_InitWrite( const char *psz_arg, size_t i_len, bool b_append )
  * stream_*: FIFO and character device handlers
  *****************************************************************************/
 static off_t i_stream_nb_skips = 0;
+static ssize_t i_buf_offset = 0;
 
 static ssize_t stream_Read( void *p_buf, size_t i_len )
 {
@@ -366,7 +378,8 @@ static ssize_t stream_Read( void *p_buf, size_t i_len )
         return 0;
     }
 
-    if ( (i_ret = read( i_input_fd, p_buf, i_len )) < 0 )
+    if ( (i_ret = read( i_input_fd, p_buf + i_buf_offset,
+                        i_len - i_buf_offset )) < 0 )
     {
         msg_Err( NULL, "read error (%s)", strerror(errno) );
         b_die = b_error = 1;
@@ -374,6 +387,13 @@ static ssize_t stream_Read( void *p_buf, size_t i_len )
     }
 
     i_stc = pf_Date();
+    i_buf_offset += i_ret;
+
+    if ( i_buf_offset < i_len )
+        return 0;
+
+    i_ret = i_buf_offset;
+    i_buf_offset = 0;
     if ( i_stream_nb_skips )
     {
         i_stream_nb_skips--;
@@ -393,6 +413,7 @@ static int stream_InitRead( const char *psz_arg, size_t i_len,
     if ( i_pos ) return -1;
 
     i_input_fd = OpenFile( psz_arg, true, false );
+    if ( i_input_fd < 0 ) return -1;
     i_stream_nb_skips = i_nb_skipped_chunks;
 
     pf_Read = stream_Read;
@@ -422,6 +443,7 @@ static void stream_ExitWrite(void)
 static int stream_InitWrite( const char *psz_arg, size_t i_len, bool b_append )
 {
     i_output_fd = OpenFile( psz_arg, false, b_append );
+    if ( i_output_fd < 0 ) return -1;
 
     pf_Write = stream_Write;
     pf_ExitWrite = stream_ExitWrite;
@@ -511,8 +533,14 @@ static int file_InitRead( const char *psz_arg, size_t i_len,
     }
 
     i_input_fd = OpenFile( psz_arg, true, false );
+    if ( i_input_fd < 0 )
+    {
+        free(psz_aux_file);
+        return -1;
+    }
     p_input_aux = OpenAuxFile( psz_aux_file, true, false );
     free( psz_aux_file );
+    if ( p_input_aux == NULL ) return -1;
 
     lseek( i_input_fd, (off_t)i_len * i_nb_skipped_chunks, SEEK_SET );
     fseeko( p_input_aux, 8 * i_nb_skipped_chunks, SEEK_SET );
@@ -572,8 +600,10 @@ static int file_InitWrite( const char *psz_arg, size_t i_len, bool b_append )
     if ( b_append )
         CheckFileSizes( psz_arg, psz_aux_file, i_len );
     i_output_fd = OpenFile( psz_arg, false, b_append );
+    if ( i_output_fd < 0 ) return -1;
     p_output_aux = OpenAuxFile( psz_aux_file, false, b_append );
     free( psz_aux_file );
+    if ( p_output_aux == NULL ) return -1;
 
     pf_Write = file_Write;
     pf_ExitWrite = file_ExitWrite;
@@ -590,29 +620,36 @@ static uint64_t i_input_dir_delay;
 
 static ssize_t dir_Read( void *p_buf, size_t i_len )
 {
-    ssize_t i_ret;
-try_again:
-    i_ret = file_Read( p_buf, i_len );
-    if ( !i_ret )
+    for ( ; ; )
     {
+        ssize_t i_ret = file_Read( p_buf, i_len );
+        if ( i_ret > 0 ) return i_ret;
+
         b_die = 0; /* we're not dead yet */
         close( i_input_fd );
         fclose( p_input_aux );
         i_input_fd = 0;
+        p_input_aux = NULL;
 
-        i_input_dir_file++;
-
-        i_input_fd = OpenDirFile( psz_input_dir_name, i_input_dir_file,
-                                  true, i_input_dir_len, &p_input_aux );
-        if ( i_input_fd < 0 )
+        for ( ; ; )
         {
-            msg_Err( NULL, "end of files reached" );
-            b_die = 1;
-            return 0;
+            i_input_dir_file++;
+
+            i_input_fd = OpenDirFile( psz_input_dir_name, i_input_dir_file,
+                                      true, i_input_dir_len, &p_input_aux );
+            if ( i_input_fd > 0 ) break;
+
+            if ( i_input_dir_file * i_rotate_size + i_rotate_offset >
+                 i_first_stc + i_duration )
+            {
+                msg_Err( NULL, "end of files reached" );
+                b_die = 1;
+                return 0;
+            }
+
+            msg_Warn( NULL, "missing segment" );
         }
-        goto try_again;
     }
-    return i_ret;
 }
 
 static bool dir_Delay(void)
@@ -633,7 +670,7 @@ static bool dir_Delay(void)
 static void dir_ExitRead(void)
 {
     free( psz_input_dir_name );
-    if ( i_input_fd )
+    if ( i_input_fd > 0 )
     {
         close( i_input_fd );
         fclose( p_input_aux );
@@ -661,23 +698,24 @@ static int dir_InitRead( const char *psz_arg, size_t i_len,
 
     psz_input_dir_name = strdup( psz_arg );
     i_input_dir_len = i_len;
-    i_input_dir_file = GetDirFile( i_rotate_size, i_pos );
+    i_input_dir_file = GetDirFile( i_rotate_size, i_rotate_offset, i_pos );
 
-    i_nb_skipped_chunks = LookupDirAuxFile( psz_input_dir_name,
-                                            i_input_dir_file, i_stc,
-                                            i_input_dir_len );
-    if ( i_nb_skipped_chunks < 0 )
+    for ( ; ; )
     {
-        /* Try at most one more chunk */
-        i_input_dir_file++;
         i_nb_skipped_chunks = LookupDirAuxFile( psz_input_dir_name,
                                                 i_input_dir_file, i_stc,
                                                 i_input_dir_len );
-        if ( i_nb_skipped_chunks < 0 )
+        if ( i_nb_skipped_chunks >= 0 ) break;
+
+        if ( i_input_dir_file * i_rotate_size + i_rotate_offset >
+             i_stc + i_duration )
         {
             msg_Err( NULL, "position not found" );
             return -1;
         }
+
+        i_input_dir_file++;
+        msg_Warn( NULL, "missing segment" );
     }
 
     i_input_fd = OpenDirFile( psz_input_dir_name, i_input_dir_file,
@@ -700,7 +738,7 @@ static uint64_t i_output_dir_file;
 
 static ssize_t dir_Write( const void *p_buf, size_t i_len )
 {
-    uint64_t i_dir_file = GetDirFile( i_rotate_size, i_stc );
+    uint64_t i_dir_file = GetDirFile( i_rotate_size, i_rotate_offset, i_stc );
     if ( !i_output_fd || i_dir_file != i_output_dir_file )
     {
         if ( i_output_fd )
@@ -872,15 +910,16 @@ static void Restamp( uint8_t *p_buffer, size_t i_read_size )
                 pes_get_streamid(p_buffer + header_size) !=
                     PES_STREAM_ID_PRIVATE_2 &&
                 pes_validate_header(p_buffer + header_size) &&
-                pes_has_pts(p_buffer + header_size) &&
-                pes_validate_pts(p_buffer + header_size)) {
+                pes_has_pts(p_buffer + header_size)
+                /* disable the check as this is a common mistake */
+                /* && pes_validate_pts(p_buffer + header_size) */) {
                 pes_set_pts(p_buffer + header_size,
                         RestampTS(pes_get_pts(p_buffer + header_size) * 300) /
                         300);
 
                 if (header_size + PES_HEADER_SIZE_PTSDTS <= TS_SIZE &&
-                    pes_has_dts(p_buffer + header_size) &&
-                    pes_validate_dts(p_buffer + header_size))
+                    pes_has_dts(p_buffer + header_size)
+                    /* && pes_validate_dts(p_buffer + header_size) */)
                     pes_set_dts(p_buffer + header_size,
                         RestampTS(pes_get_dts(p_buffer + header_size) * 300) /
                         300);
@@ -903,7 +942,6 @@ int main( int i_argc, char **pp_argv )
     int i_stc_fd = -1;
     off_t i_skip_chunks = 0, i_nb_chunks = -1;
     int64_t i_seek = 0;
-    uint64_t i_duration = 0;
     bool b_append = false;
     uint8_t *p_buffer, *p_read_buffer;
     size_t i_max_read_size, i_max_write_size;
@@ -912,7 +950,7 @@ int main( int i_argc, char **pp_argv )
     sigset_t set;
 
     /* Parse options */
-    while ( (c = getopt( i_argc, pp_argv, "i:l:t:XT:fp:CPs:n:k:d:ar:S:uUm:R:wh" )) != -1 )
+    while ( (c = getopt( i_argc, pp_argv, "i:l:t:XT:fp:CPs:n:k:d:ar:O:S:uUm:R:wh" )) != -1 )
     {
         switch ( c )
         {
@@ -980,6 +1018,10 @@ int main( int i_argc, char **pp_argv )
             i_rotate_size = strtoull( optarg, NULL, 0 );
             break;
 
+        case 'O':
+            i_rotate_offset = strtoull( optarg, NULL, 0 );
+            break;
+
         case 'S':
         {
             struct in_addr maddr;
