@@ -1,8 +1,8 @@
 /*****************************************************************************
  * multicat_validate.c: validate position in directory input
  *****************************************************************************
- * Copyright (C) 2009, 2011, 2015 VideoLAN
- * $Id: multicat_validate.c 64 2015-07-15 20:47:39Z massiot $
+ * Copyright (C) 2009, 2011, 2015-2016 VideoLAN
+ * $Id$
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -41,6 +41,7 @@
  * Local declarations
  *****************************************************************************/
 static uint64_t i_rotate_size = DEFAULT_ROTATE_SIZE;
+static uint64_t i_rotate_offset = DEFAULT_ROTATE_OFFSET;
 static int64_t i_tolerance = DEFAULT_TOLERANCE;
 static size_t i_asked_payload_size = DEFAULT_PAYLOAD_SIZE;
 static int64_t i_delay = 0;
@@ -50,9 +51,10 @@ static bool b_status = false;
 
 static void usage(void)
 {
-    msg_Raw( NULL, "Usage: multicat_validate [-l <syslogtag>] [-k <start time>] [-r <file duration>] [-W <tolerance>] [-m <payload size>] <input directory>" );
+    msg_Raw( NULL, "Usage: multicat_validate [-l <syslogtag>] [-k <start time>] [-r <file duration>] [-O <rotate offset>] [-W <tolerance>] [-m <payload size>] <input directory>" );
     msg_Raw( NULL, "    -k: start at the given position (in 27 MHz units, negative = from the end)" );
     msg_Raw( NULL, "    -r: in directory mode, rotate file after this duration (default: 97200000000 ticks = 1 hour)" );
+    msg_Raw( NULL, "    -O: in directory mode, rotate file after duration + this offset (default: 0 tick = calendar hour)" );
     msg_Raw( NULL, "    -W: maximum tolerated wait time before the forthcoming packet (by default: 27000000 ticks = 1 second)" );
     msg_Raw( NULL, "    -m: size of the payload chunk, excluding optional RTP header (default 1316)" );
     exit(EXIT_FAILURE);
@@ -106,7 +108,7 @@ int main( int i_argc, char **pp_argv )
 
     setvbuf(stdout, NULL, _IOLBF, 0);
 
-    while ( (c = getopt( i_argc, pp_argv, "l:k:r:W:m:h" )) != -1 )
+    while ( (c = getopt( i_argc, pp_argv, "l:k:r:O:W:m:h" )) != -1 )
     {
         switch ( c )
         {
@@ -122,6 +124,10 @@ int main( int i_argc, char **pp_argv )
             i_rotate_size = strtoull( optarg, NULL, 0 );
             break;
 
+        case 'O':
+            i_rotate_offset = strtoull( optarg, NULL, 0 );
+            break;
+
         case 'W':
             i_tolerance = strtoull( optarg, NULL, 0 );
             break;
@@ -155,7 +161,7 @@ int main( int i_argc, char **pp_argv )
         i_delay = real_Date() - i_stc;
     }
 
-    i_dir_file = GetDirFile( i_rotate_size, i_stc );
+    i_dir_file = GetDirFile( i_rotate_size, i_rotate_offset, i_stc );
     i_nb_skipped_chunks = LookupDirAuxFile( psz_dir_name, i_dir_file, i_stc,
                                             i_asked_payload_size );
     if ( i_nb_skipped_chunks < 0 )
