@@ -1,8 +1,8 @@
 /*****************************************************************************
  * util.c: Utils for the multicat suite
  *****************************************************************************
- * Copyright (C) 2004, 2009, 2011, 2015 VideoLAN
- * $Id: util.c 66 2015-07-17 16:03:17Z massiot $
+ * Copyright (C) 2004, 2009, 2011, 2015-2016 VideoLAN
+ * $Id$
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -21,6 +21,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
+#define _GNU_SOURCE
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdint.h>
@@ -262,7 +264,7 @@ void real_Sleep( uint64_t i_delay )
  *****************************************************************************/
 static int GetInterfaceIndex( const char *psz_name )
 {
-#if !defined(__FreeBSD__) && !defined(__APPLE__)
+#if !defined(__APPLE__)
     int i_fd;
     struct ifreq ifr;
 
@@ -283,7 +285,11 @@ static int GetInterfaceIndex( const char *psz_name )
 
     close( i_fd );
 
+#if defined(__FreeBSD__)
+    return ifr.ifr_index;
+#else
     return ifr.ifr_ifindex;
+#endif
 #else
     return 0;
 #endif
@@ -445,7 +451,11 @@ static void RawFillHeaders(struct udprawpkt *dgram,
                         uint8_t ttl, uint8_t tos, uint16_t len)
 {
 #ifndef __APPLE__
+#if defined(__FreeBSD__)
+    struct ip *iph = &(dgram->iph);
+#else
     struct iphdr *iph = &(dgram->iph);
+#endif
     struct udphdr *udph = &(dgram->udph);
 
 #ifdef DEBUG_SOCKET
@@ -458,6 +468,26 @@ static void RawFillHeaders(struct udprawpkt *dgram,
     printf("Filling raw header (%p) (%s:%u -> %s:%u)\n", dgram, ipsrc_str, portsrc, ipdst_str, portdst);
 #endif
 
+#if defined(__FreeBSD__)
+    // Fill ip header
+    iph->ip_hl    = 5;              // ip header with no specific option
+    iph->ip_v     = 4;
+    iph->ip_tos   = tos;
+    iph->ip_len   = sizeof(struct udprawpkt) + len; // auto-htoned ?
+    iph->ip_id    = htons(0);       // auto-generated if frag_off (flags) = 0 ?
+    iph->ip_off   = 0;
+    iph->ip_ttl   = ttl;
+    iph->ip_p     = IPPROTO_UDP;
+    iph->ip_sum   = 0;
+    iph->ip_src.s_addr = ipsrc;
+    iph->ip_dst.s_addr = ipdst;
+
+    // Fill udp header
+    udph->uh_sport = htons(portsrc);
+    udph->uh_dport = htons(portdst);
+    udph->uh_ulen  = htons(sizeof(struct udphdr) + len);
+    udph->uh_sum   = 0;
+#else
     // Fill ip header
     iph->ihl      = 5;              // ip header with no specific option
     iph->version  = 4;
@@ -472,17 +502,11 @@ static void RawFillHeaders(struct udprawpkt *dgram,
     iph->daddr    = ipdst;
 
     // Fill udp header
-    #ifdef __FAVOR_BSD
-    udph->uh_sport = htons(portsrc);
-    udph->uh_dport = htons(portdst);
-    udph->uh_ulen  = htons(sizeof(struct udphdr) + len);
-    udph->uh_sum   = 0;
-    #else
     udph->source = htons(portsrc);
     udph->dest   = htons(portdst);
     udph->len    = htons(sizeof(struct udphdr) + len);
     udph->check  = 0;
-    #endif
+#endif
 
     // Compute ip header checksum. Computed by kernel when frag_off = 0 ?
     //iph->check = csum((unsigned short *)iph, sizeof(struct iphdr));
@@ -531,6 +555,9 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
     in_addr_t i_raw_srcaddr = INADDR_ANY; 
     int i_raw_srcport = 0;
     char *psz_ifname = NULL;
+#ifdef __FreeBSD__
+    int hincl = 1;
+#endif
 
     bind_addr.ss.ss_family = AF_UNSPEC;
     connect_addr.ss.ss_family = AF_UNSPEC;
@@ -685,6 +712,13 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
                 i_raw_srcaddr, connect_addr.sin.sin_addr.s_addr, i_raw_srcport,
                 ntohs(connect_addr.sin.sin_port), i_ttl, i_tos, 0);
             i_fd = socket( AF_INET, SOCK_RAW, IPPROTO_RAW );
+#ifdef __FreeBSD__
+            if ( setsockopt( i_fd, IPPROTO_IP, IP_HDRINCL, &hincl, sizeof(hincl)) == -1 )
+            {
+                msg_Err( NULL, "unable to set socket (%s)", strerror(errno) );
+                exit(EXIT_FAILURE);
+            }
+#endif
         } else
             i_fd = socket( i_family, *pb_tcp ? SOCK_STREAM : SOCK_DGRAM, 0);
 
@@ -715,7 +749,7 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
 
             if ( bind_addr.ss.ss_family != AF_UNSPEC )
             {
-                #ifndef __APPLE__
+                #if !defined(__APPLE__) && !defined(__FreeBSD__)
                 if ( IN6_IS_ADDR_MULTICAST( &bind_addr.sin6.sin6_addr ) )
                 {
                     struct ipv6_mreq imr;
@@ -956,7 +990,7 @@ int OpenFile( const char *psz_arg, bool b_read, bool b_append )
         {
             msg_Err( NULL, "file %s doesn't exist (%s)", psz_arg,
                      strerror(errno) );
-            exit(EXIT_FAILURE);
+            return -1;
         }
         i_mode |= O_CREAT;
     }
@@ -971,7 +1005,7 @@ int OpenFile( const char *psz_arg, bool b_read, bool b_append )
     if ( (i_fd = open( psz_arg, i_mode, 0644 )) < 0 )
     {
         msg_Err( NULL, "couldn't open file %s (%s)", psz_arg, strerror(errno) );
-        exit(EXIT_FAILURE);
+        return -1;
     }
 
     return i_fd;
@@ -1088,7 +1122,7 @@ off_t LookupAuxFile( const char *psz_arg, int64_t i_wanted, bool b_absolute )
     munmap( p_aux, stc_stat.st_size );
     close( i_stc_fd );
 
-    return i_offset2;
+    return i_offset1;
 }
 
 /*****************************************************************************
@@ -1145,14 +1179,15 @@ void CheckFileSizes( const char *psz_file, const char *psz_aux_file,
 /*****************************************************************************
  * GetDirFile: return the prefix of the file according to the STC
  *****************************************************************************/
-uint64_t GetDirFile( uint64_t i_rotate_size, int64_t i_wanted )
+uint64_t GetDirFile( uint64_t i_rotate_size, uint64_t i_rotate_offset,
+                     int64_t i_wanted )
 {
     if ( i_wanted <= 0 )
         i_wanted += real_Date();
     if ( i_wanted <= 0 )
         return 0;
 
-    return i_wanted / i_rotate_size;
+    return (i_wanted - i_rotate_offset) / i_rotate_size;
 }
 
 /*****************************************************************************
